@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2010 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -34,6 +34,7 @@
 #include "gmime-stream-mem.h"
 #include "gmime-table-private.h"
 #include "gmime-parse-utils.h"
+#include "gmime-events.h"
 
 
 /**
@@ -45,14 +46,7 @@
  * A #GMimeMessage represents an rfc822 message.
  **/
 
-
-typedef void (* EventCallback) (gpointer sender, gpointer args);
-
-extern void _internet_address_list_block_event_handler (InternetAddressList *list, EventCallback callback, gpointer user_data);
-extern void _internet_address_list_unblock_event_handler (InternetAddressList *list, EventCallback callback, gpointer user_data);
-extern void _internet_address_list_add_event_handler (InternetAddressList *list, EventCallback callback, gpointer user_data);
-extern void _internet_address_list_remove_event_handler (InternetAddressList *list, EventCallback callback, gpointer user_data);
-
+extern GMimeEvent *_g_mime_header_list_get_changed_event (GMimeHeaderList *headers);
 
 static void g_mime_message_class_init (GMimeMessageClass *klass);
 static void g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass);
@@ -66,6 +60,7 @@ static const char *message_get_header (GMimeObject *object, const char *header);
 static gboolean message_remove_header (GMimeObject *object, const char *header);
 static char *message_get_headers (GMimeObject *object);
 static ssize_t message_write_to_stream (GMimeObject *object, GMimeStream *stream);
+static void message_encode (GMimeObject *object, GMimeEncodingConstraint constraint);
 
 static ssize_t write_structured (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_addrspec (GMimeStream *stream, const char *name, const char *value);
@@ -73,20 +68,21 @@ static ssize_t write_received (GMimeStream *stream, const char *name, const char
 static ssize_t write_subject (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_msgid (GMimeStream *stream, const char *name, const char *value);
 
-static void to_list_changed (InternetAddressList *list, GMimeMessage *message);
-static void cc_list_changed (InternetAddressList *list, GMimeMessage *message);
-static void bcc_list_changed (InternetAddressList *list, GMimeMessage *message);
+static void to_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message);
+static void cc_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message);
+static void bcc_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message);
 
 
 static GMimeObjectClass *parent_class = NULL;
 
+
 static struct {
 	const char *name;
-	EventCallback changed_cb;
+	GMimeEventCallback changed_cb;
 } recipient_types[] = {
-	{ "To",  (EventCallback) to_list_changed  },
-	{ "Cc",  (EventCallback) cc_list_changed  },
-	{ "Bcc", (EventCallback) bcc_list_changed }
+	{ "To",  (GMimeEventCallback) to_list_changed  },
+	{ "Cc",  (GMimeEventCallback) cc_list_changed  },
+	{ "Bcc", (GMimeEventCallback) bcc_list_changed }
 };
 
 #define N_RECIPIENT_TYPES G_N_ELEMENTS (recipient_types)
@@ -146,6 +142,14 @@ g_mime_message_class_init (GMimeMessageClass *klass)
 	object_class->get_header = message_get_header;
 	object_class->get_headers = message_get_headers;
 	object_class->write_to_stream = message_write_to_stream;
+	object_class->encode = message_encode;
+}
+
+static void
+mime_part_headers_changed (GMimeHeaderList *headers, gpointer args, GMimeMessage *message)
+{
+	/* clear the message's header stream */
+	g_mime_header_list_set_stream (((GMimeObject *) message)->headers, NULL);
 }
 
 static void
@@ -155,7 +159,7 @@ connect_changed_event (GMimeMessage *message, GMimeRecipientType type)
 	
 	list = message->recipients[type];
 	
-	_internet_address_list_add_event_handler (list, recipient_types[type].changed_cb, message);
+	g_mime_event_add (list->priv, recipient_types[type].changed_cb, message);
 }
 
 static void
@@ -165,7 +169,7 @@ disconnect_changed_event (GMimeMessage *message, GMimeRecipientType type)
 	
 	list = message->recipients[type];
 	
-	_internet_address_list_remove_event_handler (list, recipient_types[type].changed_cb, message);
+	g_mime_event_remove (list->priv, recipient_types[type].changed_cb, message);
 }
 
 static void
@@ -175,7 +179,7 @@ block_changed_event (GMimeMessage *message, GMimeRecipientType type)
 	
 	list = message->recipients[type];
 	
-	_internet_address_list_block_event_handler (list, recipient_types[type].changed_cb, message);
+	g_mime_event_block (list->priv, recipient_types[type].changed_cb, message);
 }
 
 static void
@@ -185,7 +189,7 @@ unblock_changed_event (GMimeMessage *message, GMimeRecipientType type)
 	
 	list = message->recipients[type];
 	
-	_internet_address_list_unblock_event_handler (list, recipient_types[type].changed_cb, message);
+	g_mime_event_unblock (list->priv, recipient_types[type].changed_cb, message);
 }
 
 static void
@@ -231,6 +235,7 @@ static void
 g_mime_message_finalize (GObject *object)
 {
 	GMimeMessage *message = (GMimeMessage *) object;
+	GMimeEvent *changed;
 	guint i;
 	
 	g_free (message->from);
@@ -249,8 +254,11 @@ g_mime_message_finalize (GObject *object)
 	g_free (message->message_id);
 	
 	/* unref child mime part */
-	if (message->mime_part)
+	if (message->mime_part) {
+		changed = _g_mime_header_list_get_changed_event (message->mime_part->headers);
+		g_mime_event_remove (changed, (GMimeEventCallback) mime_part_headers_changed, message);
 		g_object_unref (message->mime_part);
+	}
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -661,7 +669,6 @@ enum {
 	HEADER_SUBJECT,
 	HEADER_DATE,
 	HEADER_MESSAGE_ID,
-	HEADER_MIME_VERSION,
 	HEADER_UNKNOWN
 };
 
@@ -674,7 +681,6 @@ static const char *message_headers[] = {
 	"Subject",
 	"Date",
 	"Message-Id",
-	"MIME-Version",
 };
 
 enum {
@@ -766,8 +772,6 @@ process_header (GMimeObject *object, int action, const char *header, const char
 		g_free (message->message_id);
 		message->message_id = g_mime_utils_decode_message_id (value);
 		break;
-	case HEADER_MIME_VERSION:
-		break;
 	default:
 		return FALSE;
 	}
@@ -780,6 +784,9 @@ message_prepend_header (GMimeObject *object, const char *header, const char *val
 {
 	GMimeMessage *message = (GMimeMessage *) object;
 	
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
+		return;
+	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
@@ -799,6 +806,9 @@ message_append_header (GMimeObject *object, const char *header, const char *valu
 {
 	GMimeMessage *message = (GMimeMessage *) object;
 	
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
+		return;
+	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
@@ -818,6 +828,9 @@ message_set_header (GMimeObject *object, const char *header, const char *value)
 {
 	GMimeMessage *message = (GMimeMessage *) object;
 	
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
+		return;
+	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
@@ -836,21 +849,18 @@ static const char *
 message_get_header (GMimeObject *object, const char *header)
 {
 	GMimeMessage *message = (GMimeMessage *) object;
-	const char *value;
+	
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
+		return "1.0";
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
-	if (g_ascii_strncasecmp ("Content-", header, 8) != 0) {
-		if ((value = GMIME_OBJECT_CLASS (parent_class)->get_header (object, header)))
-			return value;
-		
-		if (!g_ascii_strcasecmp ("MIME-Version", header))
-			return "1.0";
-	} else if (message->mime_part) {
+	if (g_ascii_strncasecmp ("Content-", header, 8) != 0)
+		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
+	else if (message->mime_part)
 		return g_mime_object_get_header (message->mime_part, header);
-	}
-	
-	return NULL;
+	else
+		return NULL;
 }
 
 static gboolean
@@ -861,6 +871,9 @@ message_remove_header (GMimeObject *object, const char *header)
 	GMimeRecipientType type;
 	guint i;
 	
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
+		return FALSE;
+	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	if (!g_ascii_strncasecmp ("Content-", header, 8)) {
@@ -944,15 +957,9 @@ message_get_headers (GMimeObject *object)
 		/* if the mime part has raw headers, then it contains the message headers as well */
 		g_mime_header_list_write_to_stream (message->mime_part->headers, stream);
 	} else {
-		/* Note: if the user changed the mime_part's headers,
-		 * we need to unset our cached header stream or it
-		 * won't reflect the changes. */
-		g_mime_header_list_set_stream (object->headers, NULL);
-		
 		g_mime_header_list_write_to_stream (object->headers, stream);
 		if (message->mime_part) {
-			if (g_mime_object_get_header (message->mime_part, "Content-Type") &&
-			    !g_mime_header_list_get (object->headers, "MIME-Version"))
+			if (g_mime_object_get_header (message->mime_part, "Content-Type"))
 				g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
 			g_mime_header_list_write_to_stream (message->mime_part->headers, stream);
 		}
@@ -974,20 +981,13 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	
 	if (message->mime_part) {
 		if (!g_mime_header_list_get_stream (message->mime_part->headers)) {
-			/* Note: if the user changed the mime_part's headers,
-			 * we need to unset our cached header stream or it
-			 * won't reflect the changes. */
-			g_mime_header_list_set_stream (object->headers, NULL);
-			
 			if ((nwritten = g_mime_header_list_write_to_stream (object->headers, stream)) == -1)
 				return -1;
 			
 			total += nwritten;
 			
-			if (!g_mime_header_list_get (object->headers, "MIME-Version")) {
-				if ((nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n")) == -1)
-					return -1;
-			}
+			if ((nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n")) == -1)
+				return -1;
 			
 			total += nwritten;
 		}
@@ -1009,6 +1009,16 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	return total;
 }
 
+static void
+message_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
+{
+	GMimeMessage *message = (GMimeMessage *) object;
+	
+	if (message->mime_part != NULL)
+		g_mime_object_encode (message->mime_part, constraint);
+}
+
+
 
 /**
  * g_mime_message_new: Create a new MIME Message object
@@ -1147,19 +1157,19 @@ sync_recipient_header (GMimeMessage *message, GMimeRecipientType type)
 }
 
 static void
-to_list_changed (InternetAddressList *list, GMimeMessage *message)
+to_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message)
 {
 	sync_recipient_header (message, GMIME_RECIPIENT_TYPE_TO);
 }
 
 static void
-cc_list_changed (InternetAddressList *list, GMimeMessage *message)
+cc_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message)
 {
 	sync_recipient_header (message, GMIME_RECIPIENT_TYPE_CC);
 }
 
 static void
-bcc_list_changed (InternetAddressList *list, GMimeMessage *message)
+bcc_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message)
 {
 	sync_recipient_header (message, GMIME_RECIPIENT_TYPE_BCC);
 }
@@ -1446,17 +1456,29 @@ g_mime_message_get_mime_part (GMimeMessage *message)
 void
 g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
 {
-	g_return_if_fail (GMIME_IS_MESSAGE (message));
-	g_return_if_fail (GMIME_IS_OBJECT (mime_part));
+	GMimeEvent *changed;
 	
-	g_object_ref (mime_part);
-	g_mime_header_list_set_stream (mime_part->headers, NULL);
+	g_return_if_fail (mime_part == NULL || GMIME_IS_OBJECT (mime_part));
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
+	
+	if (message->mime_part == mime_part)
+		return;
 	
 	if (message->mime_part) {
+		changed = _g_mime_header_list_get_changed_event (message->mime_part->headers);
+		g_mime_event_remove (changed, (GMimeEventCallback) mime_part_headers_changed, message);
+		
 		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
 		g_object_unref (message->mime_part);
 	}
 	
+	if (mime_part) {
+		changed = _g_mime_header_list_get_changed_event (mime_part->headers);
+		g_mime_header_list_set_stream (mime_part->headers, NULL);
+		g_mime_event_add (changed, (GMimeEventCallback) mime_part_headers_changed, message);
+		g_object_ref (mime_part);
+	}
+	
 	g_mime_header_list_set_stream (((GMimeObject *) message)->headers, NULL);
 	
 	message->mime_part = mime_part;
