@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+/*  GMime
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -24,16 +24,16 @@
 
 G_BEGIN_DECLS
 
-gboolean g_mime_parse_content_type (const char **in, char **type, char **subtype);
+G_GNUC_INTERNAL gboolean g_mime_parse_content_type (const char **in, char **type, char **subtype);
 
-void g_mime_decode_lwsp (const char **in);
+G_GNUC_INTERNAL void g_mime_decode_lwsp (const char **in);
 #define decode_lwsp(in) g_mime_decode_lwsp (in)
 
-const char *g_mime_decode_word (const char **in);
+G_GNUC_INTERNAL const char *g_mime_decode_word (const char **in);
 #define decode_word(in) g_mime_decode_word (in)
 
-char *g_mime_decode_domain (const char **in);
-#define decode_domain(in) g_mime_decode_domain (in)
+G_GNUC_INTERNAL gboolean g_mime_decode_domain (const char **in, GString *domain);
+#define decode_domain(in, domain) g_mime_decode_domain (in, domain)
 
 G_END_DECLS
 
